﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.DataAnnotations;

namespace CrmDemoCodeFirst {

  /// <summary>
  /// Data Annotations attributes can be used for mapping customization. 
  /// We have used them for autoincrement column definition in primary key 
  /// and for definition of non nullable columns. 
  /// Also you can use these attributes for the MaxLength limitation of string field. 
  /// Similarly, we can do the same using the fluent mapping 
  /// in OnModelCreating method of DbContext. 
  /// 
  /// Also, we will not set up table schema name explicitly using Table attribute – 
  /// we will define it for all tables a bit later using our own convention.
  /// 
  /// Since entity properties that represent foreign keys are defined with virtual modifier, 
  /// we will use dynamic proxies for POCO classes that give us an ability of Lazy Loading usage. 
  /// If you are not going to use Lazy Loading or don’t want to get dynamic proxies, 
  /// don’t define properties with virtual modifier and use explicit Eager Loading (Include method).
  /// </summary>

  [Table("Product", SchemaName = "TEST")]
  public class Product {

    [Key]
    [DatabaseGenerated(DatabaseGenerationOption.Identity)]
    public long ProductID { get; set; }

    [Required]
    [MaxLength(50)]
    public string ProductName { get; set; }

    public string UnitName { get; set; }
    public int UnitScale { get; set; }
    public long InStock { get; set; }
    public double Price { get; set; }
    public double DiscontinuedPrice { get; set; }

    public virtual ProductCategory Category { get; set; }
    public virtual ICollection<OrderDetail> OrderDetails { get; set; }
  }

  [Table("ProductCategory", SchemaName = "TEST")]
  public class ProductCategory {

    [Key]
    [DatabaseGenerated(DatabaseGenerationOption.Identity)]
    public long CategoryID { get; set; }

    [Required]
    [MaxLength(20)]
    public string CategoryName { get; set; }

    public virtual ProductCategory ParentCategory { get; set; }
    public virtual ICollection<ProductCategory> ChildCategories { get; set; }
    public virtual ICollection<Product> Products { get; set; }
  }

  [Table("Order Details", SchemaName = "TEST")]
  public class OrderDetail {

    [Key]
    [Column(Order = 1)]
    public long OrderID { get; set; }

    [Key]
    [Column(Order = 2)]
    public long ProductID { get; set; }

    public double Price { get; set; }
    public double Quantity { get; set; }

    public virtual Product Product { get; set; }
    public virtual Order Order { get; set; }
  }

  [Table("Orders", SchemaName = "TEST")]
  public class Order {

    public Order() {

      OrderDate = DateTime.Now;
    }

    [Key]
    [DatabaseGenerated(DatabaseGenerationOption.Identity)]
    public long OrderID { get; set; }

    [Required]
    public DateTime OrderDate { get; set; }

    public double Freight { get; set; }
    public DateTime? ShipDate { get; set; }
    public Double Discount { get; set; }

    public virtual ICollection<OrderDetail> OrderDetails { get; set; }
    public virtual PersonContact PersonContact { get; set; }
    public virtual Company Company { get; set; }
    public virtual Company ShipCompany { get; set; }
  }

  [Table("Company", SchemaName = "TEST")]
  public class Company {

    public Company() {

      Address = new AddressType();
    }

    [Key]
    [DatabaseGenerated(DatabaseGenerationOption.Identity)]
    public long CompanyID { get; set; }

    [Required]
    [MaxLength(40)]
    public string CompanyName { get; set; }

    [MaxLength(100)]
    public string Web { get; set; }

    [MaxLength(50)]
    public string Email { get; set; }

    [Required]
    public AddressType Address { get; set; }

    public virtual ICollection<Order> Orders { get; set; }
    public virtual ICollection<Order> ShippedOrders { get; set; }
    public virtual PersonContact PrimaryContact { get; set; }
    public virtual ICollection<PersonContact> Contacts { get; set; }
  }

  [Table("Person Contact", SchemaName = "TEST")]
  public class PersonContact {

    public PersonContact() {
      Address = new AddressType();
    }

    [Key]
    [DatabaseGenerated(DatabaseGenerationOption.Identity)]
    public long ContactID { get; set; }

    [MaxLength(8)]
    public string Title { get; set; }

    [MaxLength(50)]
    public string FirstName { get; set; }

    [MaxLength(50)]
    public string MiddleName { get; set; }

    [MaxLength(50)]
    public string LastName { get; set; }

    [MaxLength(25)]
    public string HomePhone { get; set; }

    [MaxLength(25)]
    public string MobilePhone { get; set; }

    [Required]
    public AddressType Address { get; set; }

    public virtual ICollection<Order> Orders { get; set; }
    public virtual Company Company { get; set; }
    public virtual ICollection<Company> Companies { get; set; }
  }

  [ComplexType]
  public class AddressType {

    [MaxLength(120)]
    public string AddressTitle { get; set; }

    [MaxLength(60)]
    public string Address { get; set; }

    [MaxLength(30)]
    public string City { get; set; }

    [MaxLength(20)]
    public string Region { get; set; }

    [MaxLength(15)]
    public string PostalCode { get; set; }

    [MaxLength(20)]
    public string Country { get; set; }

    [MaxLength(25)]
    public string Phone { get; set; }

    [MaxLength(25)]
    public string Fax { get; set; }
  }
}
